#ifndef VFH_NEAREST_NEIGHBOR_H
#define VFH_NEAREST_NEIGHBOR_H

#include <iostream>
#include <fstream>
#include <string>
#include <vector>

#include <Eigen/Dense>
#include <Eigen/Geometry>

#include <flann/flann.h>
#include <flann/io/hdf5.h>

#include <boost/thread/thread.hpp>
#include <boost/filesystem.hpp>

#include <pcl/features/vfh.h>
#include <pcl/point_types.h>
#include <pcl/io/pcd_io.h>
#include <pcl/registration/icp.h>
#include <pcl/common/transforms.h>
#include "PCLRegistration.h"

#include <Eigen/Eigen>

using namespace std;

typedef std::pair<std::string, std::vector<float> > vfh_model;

class VFHNearestNeighbor
{
  public:
    pcl::PointCloud<pcl::PointXYZ>::Ptr cloud_obj;
    string neighbor_model_name;
    Eigen::Matrix4d icp_matrix_;

    static inline void nearestKSearch (flann::Index<flann::ChiSquareDistance<float> > &index, const vfh_model &model,
      int k, flann::Matrix<int> &indices, flann::Matrix<float> &distances);
    static bool loadFileList (std::vector<vfh_model> &models, const std::string &filename);
    int estimate6DOF (std::vector< pcl::PointCloud<pcl::VFHSignature308> > &vfh_vector,
      vector< pcl::PointCloud<pcl::PointXYZ> > &cluster_scene_vector);
};

#endif
