#include "../include/Estimate6DOF.h"

#define DEBUG
#define _DEBUG

using namespace std;
string Estimate6DOF::numberToString (float x, float y, float z,
  float q1, float q2, float q3, float q4)
{
  char xyzq1234[256];

  sprintf (xyzq1234, "%.3f;%.3f;%.3f;%.5f;%.5f;%.5f;%.5f" ,x ,y ,z ,q1 ,q2 ,q3, q4);

  return string (xyzq1234);
}

void Estimate6DOF::print4x4Matrix (const Eigen::Matrix4d &matrix, bool flag_milimeter)
{
  float f = 1000;

  printf ("Rotation matrix :\n");
  printf ("    | %6.4f %6.4f %6.4f | \n", matrix (0, 0), matrix (0, 1), matrix (0, 2));
  printf ("R = | %6.4f %6.4f %6.4f | \n", matrix (1, 0), matrix (1, 1), matrix (1, 2));
  printf ("    | %6.4f %6.4f %6.4f | \n", matrix (2, 0), matrix (2, 1), matrix (2, 2));
  printf ("Translation vector :\n");

  if (flag_milimeter == false)
    printf ("t = < %6.4f, %6.4f, %6.4f >\n\n", matrix (0, 3), matrix (1, 3), matrix (2, 3));
  else
    printf ("t = < %6.2f, %6.2f, %6.2f >\n\n", f*matrix (0, 3), f*matrix (1, 3), f*matrix (2, 3));
}

Mat Estimate6DOF::InputMatrix (const std::string &pose_filename)
{
  ifstream in (pose_filename.c_str() );
  Mat matrix;
  std::istringstream istr;
  std::string str;
  std::vector<double> tmpvec;

  while (getline(in,str))
  {
    istr.str (str);
    double tmp;
    while (istr >> tmp)
    {
      tmpvec.push_back (tmp);
    }
    matrix.push_back (tmpvec);
    tmpvec.clear ();
    istr.clear ();
  }

  in.close ();
  return matrix;
}

Eigen::Matrix4d Estimate6DOF::angle2pose(int angle_z, int angle_y)
{
    double radian_z= angle_z*M_PI/180;
    double radian_y= angle_y*M_PI/180;
    Eigen::Matrix3d rot_z;
    Eigen::Matrix3d rot_y;
    Eigen::Matrix3d obj2camera;
    Eigen::Matrix3d camera2obj;
    Eigen::Matrix3d blensor2real;
    Eigen::Matrix4d pose;

    rot_z<< cos(radian_z),-sin(radian_z),0,
            sin(radian_z),cos(radian_z),0,
            0,0,1;
    rot_y<<cos(radian_y),0,sin(radian_y),
            0,1,0,
            -sin(radian_y),0,cos(radian_y);

    blensor2real<<0,1,0,
            1,0,0,
            0,0,-1;//Z270,Y180 in obj frame
    obj2camera = rot_z*rot_y*blensor2real;
    camera2obj = obj2camera.inverse();
    Eigen::Vector4d trans(0,0,1,1);
    for (int i=0;i<4;++i)
        for (int j=0;j<3;++j)
        {
            if (i==3)
                pose(i,j) = 0;
            else
                pose(i,j) = camera2obj(i,j);
        }
    pose.col(3)=trans;
#ifdef DEBUG
std::cout<<"template pose in camera:"<<std::endl<<pose<<std::endl;
#endif // DEBUG
    return pose;
}
/*
int Estimate6DOF::estimate6DOF(pcl::PointCloud<pcl::PointXYZ>::Ptr &nearest_model_cloud,pcl::PointCloud<pcl::PointXYZ>::Ptr &cloud_tgt, Eigen::Vector4f &position_base_at_camera, Eigen::Matrix4f &camera_to_robot)
{
  //load pose
  std::stringstream pose_filename;
  pose_filename << "pose_matrix/pose_" << pose_number <<".txt";
  Mat pose_matrix_array = InputMatrix (pose_filename.str().c_str());
  Eigen::Matrix4f pose_matrix;

  for (int m = 0; m < pose_matrix_array.size();m++)
    for (int n = 0; n < pose_matrix_array[m].size();n++)
      pose_matrix(m,n) = pose_matrix_array[m][n];

//-------pre-align-------
  Eigen::Vector4f object_center, nearest_model_center, transform_t;
  Eigen::Matrix4f pre_transfrom_matrix;
  pcl::compute3DCentroid (*nearest_model_cloud, nearest_model_center);
  pcl::compute3DCentroid (*target_cluster, object_center);
  transform_t = object_center - nearest_model_center;

  for (int m = 0; m < 4; m++)
    for (int n = 0; n < 4; n++)
    {
      if (n == 3)
        pre_transfrom_matrix(m,n) = transform_t[m];
      else if (m == n)
        pre_transfrom_matrix(m,n) = 1;
      else
        pre_transfrom_matrix(m,n) = 0;
    }
#ifdef DEBUG
print4x4Matrix (pre_transfrom_matrix);
#endif // DEBUG

  pcl::transformPointCloud (*nearest_model_cloud, *cloud, pre_transfrom_matrix);

//---------------------------align-------------------------------------
  // Object for storing the align point cloud.
  pcl::PointCloud<pcl::PointXYZ>::Ptr cloud_aligned (new pcl::PointCloud<pcl::PointXYZ>);
  pcl::IterativeClosestPoint<pcl::PointXYZ, pcl::PointXYZ> align;
#ifdef _DEBUG
  pcl::io::savePCDFileASCII ("source_cloud.pcd", *cloud);
  pcl::io::savePCDFileASCII ("target_cloud.pcd", *target_cluster);
#endif // DEBUG
  align.setInputSource (cloud);
  align.setInputTarget (target_cluster);

  //Set the max correspondence distance to x(m) (e.g., correspondences with higher distances will be ignored)
  align.setMaxCorrespondenceDistance (0.3);
  //Set the maximum number of iterations (criterion 1)
  align.setMaximumIterations (200);
  // Set the transformation epsilon (criterion 2)
  align.setTransformationEpsilon(1e-14);
  // Set the euclidean distance difference epsilon (criterion 3)
  align.setEuclideanFitnessEpsilon(0.001);
  align.align (*cloud_aligned);
  align_transformation = align.getFinalTransformation ();
#ifdef DEBUG
print4x4Matrix (align_transformation);
#endif // DEBUG
//----------------------save the matrix, get the centroid point------------------------------------------------------------
  if (align.hasConverged ())
  {
    for (int k = 0; k < 3; k++)
      align_transformation(k,3) = align_transformation(k,3) + transform_t[k];

    transformation = camera_to_robot * align_transformation * pose_matrix;

    //the R and T matrix from matrix to quaternion
    Eigen::Matrix3f matrix_to_q;
    for (int mi = 0; mi < 3; mi++)
      for (int ni = 0; ni < 3; ni++)
        matrix_to_q(mi, ni) = transformation (mi, ni);

    Eigen::Quaternionf q (matrix_to_q);
    q_[0] = q.w();
    q_[1] = q.x();
    q_[2] = q.y();
    q_[3] = q.z();

    Eigen::Vector4f object_origin_at_robot;
    object_origin_at_robot = camera_to_robot * align_transformation * position_base_at_camera;

    xyz_[0] = 1000 * object_origin_at_robot[0];
    xyz_[1] = 1000 * object_origin_at_robot[1];
    xyz_[2] = 1000 * object_origin_at_robot[2];

    float roll, pitch, yaw;
    Eigen::Affine3f transformation_3f;
    transformation_3f = transformation;
    pcl::getEulerAngles (transformation_3f, roll, pitch, yaw);
  }
  else
    cout << "Failed to align" << endl;

  return (0);
}
*/
