#include "../include/PCLFilters.h"

//----------removeNaN---------------------
void RemoveNaN::removeNaN (pcl::PointCloud<pcl::PointXYZ> &cloud_in, pcl::PointCloud<pcl::PointXYZ> &cloud_out)
{
  std::vector<int> mapping;
  pcl::removeNaNFromPointCloud (cloud_in, cloud_out, mapping);
}

//---------PassThroughFilter---------------
void PassThroughFilter::filter (pcl::PointCloud<pcl::PointXYZ>::Ptr &cloud,
      pcl::PointCloud<pcl::PointXYZ>::Ptr &cloud_filtered, char direction, double d1, double d2, bool outside)
{
  // Create the filtering object
  pcl::PassThrough<pcl::PointXYZ> pass;
  pass.setInputCloud (cloud);

  if (direction == 'x')
  {
    pass.setFilterFieldName ("x");
    pass.setFilterLimits (d1, d2);
  }
  else if (direction == 'y')
  {
    pass.setFilterFieldName ("y");
    pass.setFilterLimits (d1, d2);
  }
  else if (direction == 'z')
  {
    pass.setFilterFieldName ("z");
    pass.setFilterLimits (d1, d2);
  }
  else
    std::cerr << "Error: The direction inputed of PassThroughFilter is wrong!" <<
    "It should be 'x' 'y' 'z'" << std::endl;

  pass.setFilterLimitsNegative (outside);
  pass.filter (*cloud_filtered);
}

//----------VoxelGridFilter----------
void VoxelGridFilter::filter (pcl::PointCloud<pcl::PointXYZ>::Ptr &cloud,
  pcl::PointCloud<pcl::PointXYZ>::Ptr &cloud_filtered,
  double l1, double l2, double l3)
{
  // Create the filtering object
  pcl::VoxelGrid<pcl::PointXYZ> sor;
  sor.setInputCloud (cloud);
  sor.setLeafSize (l1, l2, l3);
  sor.filter (*cloud_filtered);
}

//-------------OutlierRemovalFilter---------------------------
void OutlierRemovalFilter::filter (pcl::PointCloud<pcl::PointXYZ>::Ptr &cloud,
  pcl::PointCloud<pcl::PointXYZ>::Ptr &cloud_filtered, int mean_k,
  double std_dev_mul_thresh, bool flag_negative)
{
  // Create the filtering object
  pcl::StatisticalOutlierRemoval<pcl::PointXYZ> sor;
  sor.setInputCloud (cloud);
  sor.setMeanK (mean_k);
  sor.setStddevMulThresh (std_dev_mul_thresh);

  if (flag_negative == true)
   sor.setNegative (true);

   sor.filter (*cloud_filtered);
}

//------------PlaneProjecting----------------------------------
void PlaneProjecting::projecting (pcl::PointCloud<pcl::PointXYZ>::Ptr &cloud,
  pcl::PointCloud<pcl::PointXYZ>::Ptr &cloud_projected,
  pcl::ModelCoefficients::Ptr &coefficients)
{
  // Create the filtering object
  pcl::ProjectInliers<pcl::PointXYZ> proj;
  proj.setModelType (pcl::SACMODEL_PLANE);
  proj.setInputCloud (cloud);
  proj.setModelCoefficients (coefficients);
  proj.filter (*cloud_projected);
}

