#include "arm_interface_youbot.h"
#include <brics_actuator/JointPositions.h>
#include <brics_actuator/JointValue.h>
#include <iostream>
#include <sstream>
#include <ros/ros.h>
////////////////////////////////////////////////////////////////////////////////
//  Construction / Destruction
////////////////////////////////////////////////////////////////////////////////

cArmInterfaceYoubot::cArmInterfaceYoubot(ros::NodeHandle& nh)
        :	mGripperWidthAtGrasp( 0.004 ),
                mGripperWidthOpen( 0.0115 )
{
	mArmJointsPub = nh.advertise<brics_actuator::JointPositions>("/arm_1/arm_controller/position_command", 1);
	mGripperPub = nh.advertise<brics_actuator::JointPositions>("/arm_1/gripper_controller/position_command", 1);

	// --- Define Arm down (下拉) --- //
	mArm_Rec_PoseVals.push_back( 5.56 );  //A1
	mArm_Rec_PoseVals.push_back( 2.55 );
	mArm_Rec_PoseVals.push_back( -2.57 );
	mArm_Rec_PoseVals.push_back( 1.91 );
	mArm_Rec_PoseVals.push_back( 0.111 );  //A5

	// --- Define Arm up --- //
	mArm_Upright_PoseVals.push_back( 3.68 );
	mArm_Upright_PoseVals.push_back( 1.35 );
	mArm_Upright_PoseVals.push_back( -2.59 );
	mArm_Upright_PoseVals.push_back( 1.6 );
	mArm_Upright_PoseVals.push_back( 0.111 );

	// --- Define Arm Place --- //

	mArm_Place_PoseVals.push_back( 2.98 );
	mArm_Place_PoseVals.push_back( 0.65 );
	mArm_Place_PoseVals.push_back( -3.60 );
	mArm_Place_PoseVals.push_back( 0.89 );
	mArm_Place_PoseVals.push_back( 2.92 );


/*
	// move arm straight up. values were determined empirically
 	jointvalues[0] = 3.68;		//A1  0.0100692-5.84014
 	jointvalues[1] = 1.35;		//A2  0.0100692-2.61799
	jointvalues[2] = -2.59;		//A3  -5.02655-0.015708
 	jointvalues[3] = 1.6;		//A4  0.0221239-3.4292
 	jointvalues[4] = 0.11;		//A5  0.110619-5.64159
	msg = createArmPositionCommand(jointvalues);
	armPublisher.publish(msg);
	ros::Duration(1).sleep();

	// move arm straight down
 	jointvalues[0] = 3.80;		//A1  0.0100692-5.84014
 	jointvalues[1] = 0.1;		//A2  0.0100692-2.61799
	jointvalues[2] = -3.61;		//A3  -5.02655-0.015708
 	jointvalues[3] = 1.6;		//A4  0.0221239-3.4292
 	jointvalues[4] = 0.11;		//A5  0.110619-5.64159
*/


}

cArmInterfaceYoubot::~cArmInterfaceYoubot()
{

}

////////////////////////////////////////////////////////////////////////////////
//  Interface Functions
////////////////////////////////////////////////////////////////////////////////

void cArmInterfaceYoubot::PublishJointValues(const std::vector<double>& newPositions)
{
    	int numberOfJoints = 5;
	brics_actuator::JointPositions pos;

//	if (newPositions.size() < numberOfJoints)
//		return 0;
//                break;
	for( std::size_t i = 0; i < newPositions.size(); ++i )
	{
		brics_actuator::JointValue val;
		val.timeStamp = ros::Time::now();
		val.value = newPositions[i];
		val.unit = "rad";
		std::ostringstream jointName;
		jointName << "arm_joint_" << (i+1);
		val.joint_uri = jointName.str();
		pos.positions.push_back(val);
	}
	mArmJointsPub.publish(pos);
}

////////////////////////////////////////////////////////////////////////////////
//  Helper Functions
////////////////////////////////////////////////////////////////////////////////

void cArmInterfaceYoubot::PublishGripperValues(double width)
{
	brics_actuator::JointPositions pos;

	brics_actuator::JointValue finger;
	finger.timeStamp = ros::Time::now();
	finger.joint_uri = "gripper_finger_joint_l";
	finger.unit = "m";
	finger.value = width;
	pos.positions.push_back(finger);

	finger.joint_uri = "gripper_finger_joint_r";
	finger.unit = "m";
	finger.value = width;
	pos.positions.push_back(finger);

	mGripperPub.publish(pos);
}

////////////////////////////////////////////////////////////////////////////////

void cArmInterfaceYoubot::OpenGrippers()
{
	PublishGripperValues( mGripperWidthOpen );
}

////////////////////////////////////////////////////////////////////////////////

void cArmInterfaceYoubot::CloseGrippers()
{
	PublishGripperValues( mGripperWidthAtGrasp );
}

void cArmInterfaceYoubot::Arm_Recognition_Ready()  //识别准备，手臂下拉
{
	PublishJointValues(mArm_Rec_PoseVals);
}

void cArmInterfaceYoubot::Arm_Upright()  //手臂竖直
{
	PublishJointValues(mArm_Upright_PoseVals);
}

void cArmInterfaceYoubot::Arm_Place()  //手臂放置
{
	PublishJointValues(mArm_Place_PoseVals);
}


void cArmInterfaceYoubot::Arm_Grasp(const std::vector<double>& marker_jointvalues,const std::vector<double>& adjust_jointvalues)  //抓取
{
	Arm_Upright();
	PublishJointValues(adjust_jointvalues);
	PublishJointValues(marker_jointvalues);



}




























